import math
import os
import re
import urllib
from xml.dom import minidom
from PIL import Image

event_name = 'ludum-dare-18'
all_entries_page_url = "http://www.ludumdare.com/compo/%s/?action=preview&etype=" % event_name
column_width = 120.0
row_height = 90.0
wallpaper_margin = 48
wallpaper_sizes = [ (1024,768),
                    (1152,864),
                    (1280,768),
                    (1280,800),
                    (1280,960),
                    (1280,1024),
                    (1366,768),
                    (1440,900),
                    (1600,900),
                    (1600,1200),
                    (1680,1050),
                    (1920,1080),
                    (1620,1200) ]
grid_slots_for_logo = 6
logo_margin = 10
                
def safefilename(title):
    r"""replace characters that are bad in filenames"""
    return re.sub(r'[\\/:\*\?<>\|"]','_',title.strip())

def resize_to_fit(image, max_width, max_height):
    r"""shrink an image to fit in the given size.  
    
    returns a new one if it had to resize or the original if it was okay"""
    w,h = image.size
    mult = min(1, max_width/float(w), max_height/float(h))
    return image.resize((int(w*mult),int(h*mult) ), Image.ANTIALIAS) if mult < 1 else image

def add_to_xml(document,id,thumb_url,thumb_path,title,author):
    r"""add a game to the xml document"""
    element = document.createElement('game')
    element.setAttribute('id',id)
    element.setAttribute('thumb_url',thumb)
    element.setAttribute('localthumb',thumb_path)
    element.setAttribute('title',title)
    element.setAttribute('author',author)
    document.documentElement.appendChild(element)

def download(url,localpath):
    r"""grab it from the interwebs if we don't already have it"""
    if not os.path.exists(localpath):
        urllib.urlretrieve(url, localpath)
        
def paste_into_grid(grid_image, thumb_path, grid_index, column_width, row_height):
    r"""open an image, resize it to match the gridsize, paste it in the slot for the given index"""
    thumb_image = Image.open(thumb_path)    
    thumb_image = resize_to_fit(thumb_image,column_width,row_height)
    w,h = thumb_image.size
    row = grid_index // columns
    column = grid_index % columns
    offset = ( int((column * column_width) + 0.5*(column_width - w)),
               int((row * row_height) + 0.5*(row_height - h)) )
    grid_image.paste( thumb_image, offset )

# action will be in group 1.
# id will be in group 2.  
# url to thumbnail will be in group 3
# game title will be in group 4
# author will be in group 5
entry_re = re.compile(r"<a href='\?action=(.*?)&uid=([0-9]+)'><img src='(.*?)'><div class='title'><i>(.*?)</i></div>(.*?)</a></div>")

f = urllib.urlopen(all_entries_page_url)
contents = f.read()
f.close()
game_matches = entry_re.findall(contents)

xml_document = minidom.getDOMImplementation().createDocument(None, event_name, None)

if not os.path.exists(event_name):
    os.mkdir(event_name)
num_entries = len(game_matches)
rows = math.ceil( math.sqrt(num_entries) )
columns = math.ceil( num_entries / rows )

# oh eff let's just hard code it (up top) so as not to get ugly space in the middle
# grid_slots_for_logo = (rows * columns) - num_entries

grid_index = 0
logo_start_index = math.floor(rows/2)*columns + math.floor((columns - grid_slots_for_logo) / 2)

grid_image = Image.new( "RGB", 
                        (int(column_width*columns), int(row_height*rows)), 
                        0 )

for i,(action, id, thumb, title, author) in enumerate(game_matches):
    #make folder
    dir = os.path.join(event_name, safefilename(author))
    if not os.path.exists(dir):
        os.mkdir(dir)
        
    thumb_path = os.path.join(dir, thumb.split('/')[-1])
    print "downloading thumb for %s's game" % author
    download(thumb, thumb_path)
    add_to_xml(xml_document,id,thumb,thumb_path,title,author)
    paste_into_grid(grid_image, thumb_path, grid_index, column_width, row_height)
    
    grid_index = grid_index + 1
    
    #skip the logo space
    if grid_index == logo_start_index:
        grid_index = grid_index + grid_slots_for_logo
    
f = open('%s.xml'%event_name,'w')
xml_document.writexml(f);

#insert ludum dare logo
if grid_slots_for_logo>0:
    ld_logo = Image.open('LD2009-HighresNoShadow.png')
    ld_logo = resize_to_fit( ld_logo, 
                             grid_slots_for_logo*column_width - 2*logo_margin, 
                             row_height - 2*logo_margin)
    w,h = ld_logo.size
    row = logo_start_index // columns
    column = logo_start_index % columns
    offset = ( int((column * column_width) + 0.5*(grid_slots_for_logo*column_width - w)),
               int((row * row_height) + 0.5*(row_height-h)) )
    grid_image.paste(ld_logo, offset)

# save grid by itself
grid_image.save('grid.png','png')

# make wallpapers out of grid image
for wall_w,wall_h in wallpaper_sizes:
    print "making %d x %d wallpaper" % (wall_w,wall_h)
    available_w = wall_w - (2 * wallpaper_margin)
    available_h = wall_h - (2 * wallpaper_margin)
    resized_grid = resize_to_fit(grid_image, available_w, available_h)
    w,h = resized_grid.size
    wallpaper = Image.new( "RGB",  (wall_w, wall_h),  0 )
    offset = ( int(0.5 * (wall_w - w)),
               int(0.5 * (wall_h - h)) )
    wallpaper.paste(resized_grid, offset)
    wallpaper.save('%s-grid-%04dx%04d.png' % (event_name, wall_w, wall_h),'png')